<?php
/*
 * @Author: Qinver
 * @Url: zibll.com
 * @Date: 2021-01-23 13:36:15
 * @LastEditTime: 2021-01-24 23:44:30
 */

//定义全局变量
global $wpdb;
$wpdb->zib_cardcode  = $wpdb->prefix . 'zib_cardcode';

class zib_cp
{
    //获取用户卡密数据
    public static function user_get($user_id, $post_id = 0)
    {
        global $wpdb;
        $prepare = array();
        $WHERE = 'WHERE `cardcode` is not null AND order_type = %s AND status= %s AND user_id = %d';
        $prepare[] = 7;
        $prepare[] = 1;
        $prepare[] = $user_id;

        if ($post_id) {
            $WHERE .= ' AND post_id = %d';
            $prepare[] = $post_id;
        }

        $pay_order = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->zibpay_order} $WHERE", $prepare));

        return $pay_order ? (array)$pay_order : false;
    }

    //为用户增加卡密数据
    public static function user_add($post_id, $order_num)
    {
        global $wpdb;
        $cp_data = self::pay($post_id, $order_num);
        if (!$cp_data) return false;

        //准备参数
        $order_data = array(
            'cardcode' => maybe_serialize($cp_data),
        );

        //准备查询参数
        $where = array('order_num' => $order_num);

        if (false !== $wpdb->update($wpdb->zibpay_order, $order_data, $where)) {
            $order = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wpdb->zibpay_order} WHERE order_num = %s AND status = %d", $order_num, 1));
            if ($order) {
                do_action('add_cardcode_success', $order);
                return $order;
            }
        }
    }

    //获取文章一个未售的卡密数据
    public static function pay($post_id, $order_num)
    {
        $cp_data = self::get($post_id);
        if (!$cp_data) return false;
        $pay_data = array();
        foreach ($cp_data as $key => $cp) {
            if (!empty($cp['code']) && empty($cp['status'])) {
                $cp['order'] = $order_num;
                $cp['modified_time'] = current_time('mysql');
                $cp['status'] = 1;
                $pay_data = $cp;
                $cp_data[$key] = $cp;
                break;
            }
        }

        if (!$pay_data) return false;
        //更新数据
        if (update_post_meta($post_id, 'cardcode', $cp_data)) {
            return $pay_data;
        }
        return false;
    }

    //用户数据格式化
    public static function user_map($data)
    {
        $defaults = array(
            'code' => '',
            'key' => '',
            'posts_id' => '',
            'pay_time' => '',
            'order' => '',
        );
        $val = wp_parse_args((array) $data, $defaults);
        if (!$val['code']) {
            return;
        }
        return $val;
    }

    //获取文章卡密数据
    public static function get($post_id)
    {
        $cp = get_post_meta($post_id, 'cardcode', true);
        return $cp ? (array)$cp : false;
    }

    //删除文章的卡密数据
    public static function delete($post_id, $key)
    {

        $data = self::get($post_id);
        $delete = array();
        if (is_array($key)) {
            //如果传入数组,则循环删除
            foreach ($key as $key_v) {
                if (!empty($data[$key_v])) {
                    $delete[$key_v] = $data[$key_v];
                    unset($data[$key_v]);
                }
            }
        } else {
            if (!empty($data[$key])) {
                $delete[$key] = $data[$key];
                unset($data[$key]);
            }
        }

        if (update_post_meta($post_id, 'cardcode', $data)) {
            return array('delete' => $delete, 'count' => count($delete));
        } else {
            return false;
        }
    }

    //更新文章卡密数据
    public static function update($post_id, $val = array())
    {
        $old = self::get($post_id);
        if (is_array($old)) {
            $old_count = count($old);
            $new = array_merge($old, $val);
        } else {
            $old_count = 0;
            $new = $val;
        }
        $new = self::array_unset_tt($new, 'code');
        $new_count = count($new);
        if (update_post_meta($post_id, 'cardcode', $new)) {
            return array('cardcode' => $new, 'add_count' => ($new_count - $old_count));
        } else {
            return false;
        }
    }

    //导入文章卡密数据
    public static function import($post_id, $val)
    {
        //分割每一行
        $row_array = preg_split("/\r|\n/", $val);

        //每一行分割为数组
        $val_array = array();
        foreach ($row_array as $row_s) {
            //如果没有卡号则跳出
            $row = preg_split("/,|，|----|\s/", $row_s);
            if (empty($row[0])) {
                continue;
            }
            $val_array[] = array(
                'code' => !empty($row[0]) ? trim($row[0]) : '',
                'key' => !empty($row[1]) ? trim($row[1]) : '',
                'status' => 0,
                'create_time' => current_time('mysql'),
                'modified_time' => '',
                'order' => '',
            );
        }
        return self::update($post_id, $val_array);
    }

    //添加文章卡密数据
    public static function add($post_id, $val)
    {
        if (is_array($val)) {
            $val = array_map(array('zib_cp', "map"), $val);
            return self::update($post_id, $val);
        } else {
            return self::import($post_id, $val);
        }
    }

    /**
     * @description:
     * @param {*} $post_id
     * @param {*} $status 全部：all|未售有效：valid|已售无效：invalid
     * @return {*}
     */
    public static function count($post_id, $status = 'all')
    {
        $data = self::get($post_id);

        if (!$data) return 0;

        if ($status == 'all') {
            $count = count($data);
        } else {
            $status_1 = array_keys(array_column($data, 'status'), 1);
            $status_1_count = $status_1 ? count($status_1) : 0;
            if ($status == 1 || $status == 'invalid') {
                $count = $status_1_count;
            } else {
                $count = count($data) - $status_1_count;
            }
        }
        return $count;
    }

    //获取用户购买的卡密数量
    public static function user_count($user_id, $post_id = 0)
    {
        global $wpdb;
        $WHERE = "WHERE `status` = 1 and `order_type` = 7 and `user_id` = $user_id and `cardcode` is not null";
        if ($post_id) {
            $WHERE .= " and `post_id` = $post_id";
        }

        $count =  $wpdb->get_var("SELECT COUNT(user_id) FROM $wpdb->zibpay_order $WHERE");
        return $count;
    }

    //数组去重
    public static function array_unset_tt($arr, $key)
    {
        //建立一个目标数组
        $res = array();
        foreach ($arr as $value) {
            //查看有没有重复项
            if (isset($res[$value[$key]])) {
                //有：销毁
                unset($value[$key]);
            } else {
                $res[$value[$key]] = $value;
            }
        }
        return $res;
    }

    //
    public static function status_name($status, $html = true)
    {
        $status = (int)$status;
        $status_array = array(
            0 => array(
                'name' => '未售',
                'class' => 'b-blue',
            ),
            1 => array(
                'name' => '已售',
                'class' => 'b-red',
            ),
        );
        $name = $status_array[$status]['name'];
        $name = $html ? '<span style="padding: 1px 10px;min-width: auto;" class="but ' . $status_array[$status]['class'] . '">' . $name . '</span>' : $name;
        return $name;
    }
    public static function map($val)
    {
        $defaults = array(
            'code' => '',
            'key' => '',
            'status' => 0,
            'create_time' => '',
            'modified_time' => '',
            'order' => '',
        );
        $val = wp_parse_args((array) $val, $defaults);
        if (!$val['code']) {
            return;
        }
        return $val;
    }

    //over
}
